@@ -320,7 +320,7 @@ def bash_completions(prefix, line, begidx, endidx, env=None, paths=None,
     endidx : int
         The index in line that prefix ends on.
     env : Mapping, optional
-        The environment dict to execute the Bash suprocess in.
+        The environment dict to execute the Bash subprocess in.
     paths : list or tuple of str or None, optional
         This is a list (or tuple) of strings that specifies where the
         ``bash_completion`` script may be found. The first valid path will
@@ -896,6 +896,7 @@ def complete_from_man(prefix, line, start, end, ctx):
 # amalgamated os
 # amalgamated re
 ast = _LazyModule.load('ast', 'ast')
+glob = _LazyModule.load('glob', 'glob')
 # amalgamated builtins
 # amalgamated xonsh.tools
 # amalgamated xonsh.platform
@@ -1155,6 +1156,7 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     env = builtins.__xonsh_env__
     csc = env.get('CASE_SENSITIVE_COMPLETIONS')
     glob_sorted = env.get('GLOB_SORTED')
+    prefix = glob.escape(prefix)
     for s in xt.iglobpath(prefix + '*', ignore_case=(not csc),
                           sort_result=glob_sorted):
         paths.add(s)
@@ -1370,7 +1372,7 @@ def python_signature_complete(prefix, line, end, ctx, filter_func):
         return set()
     funcname = xt.subexpr_before_unbalanced(front, '(', ')')
     val, _ctx = _safe_eval(funcname, ctx)
-    if val is None and _ctx is None:
+    if val is None:
         return set()
     try:
         sig = inspect.signature(val)
@@ -1502,6 +1504,7 @@ def default_completers():
         ('completer', complete_completer),
         ('skip', complete_skipper),
         ('pip', complete_pip),
+        ('xpip', complete_pip),
         ('cd', complete_cd),
         ('rmdir', complete_rmdir),
         ('xonfig', complete_xonfig),
