@@ -99,16 +99,16 @@ def _dynamically_collapsed_pwd():
     """Return the compact current working directory.  It respects the
     environment variable DYNAMIC_CWD_WIDTH.
     """
-    originial_path = _replace_home_cwd()
+    original_path = _replace_home_cwd()
     target_width, units = builtins.__xonsh_env__['DYNAMIC_CWD_WIDTH']
     elision_char = builtins.__xonsh_env__['DYNAMIC_CWD_ELISION_CHAR']
     if target_width == float('inf'):
-        return originial_path
+        return original_path
     if (units == '%'):
         cols, _ = shutil.get_terminal_size()
         target_width = (cols * target_width) // 100
     sep = xt.get_sep()
-    pwd = originial_path.split(sep)
+    pwd = original_path.split(sep)
     last = pwd.pop()
     remaining_space = target_width - len(last)
     # Reserve space for separators
@@ -160,7 +160,7 @@ def env_name(pre_chars='(', post_chars=')'):
 
 
 def vte_new_tab_cwd():
-    """This prints an escape squence that tells VTE terminals the hostname
+    """This prints an escape sequence that tells VTE terminals the hostname
     and pwd. This should not be needed in most cases, but sometimes is for
     certain Linux terminals that do not read the PWD from the environment
     on startup. Note that this does not return a string, it simply prints
@@ -190,11 +190,33 @@ GitStatus = collections.namedtuple('GitStatus',
 
 def _check_output(*args, **kwargs):
     kwargs.update(dict(env=builtins.__xonsh_env__.detype(),
+                       stdout=subprocess.PIPE,
                        stderr=subprocess.DEVNULL,
-                       timeout=builtins.__xonsh_env__['VC_BRANCH_TIMEOUT'],
                        universal_newlines=True
                        ))
-    return subprocess.check_output(*args, **kwargs)
+    timeout = builtins.__xonsh_env__['VC_BRANCH_TIMEOUT']
+    # See https://docs.python.org/3/library/subprocess.html#subprocess.Popen.communicate
+    with subprocess.Popen(*args, **kwargs) as proc:
+        try:
+            out, err = proc.communicate(timeout=timeout)
+            if proc.returncode != 0:
+                raise subprocess.CalledProcessError(proc.returncode, proc.args, output=out, stderr=err)  # note err will always be empty as we redirect stderr to DEVNULL abvoe
+            return out
+        except subprocess.TimeoutExpired:
+            # We use `.terminate()` (SIGTERM) instead of `.kill()` (SIGKILL) here
+            # because otherwise we guarantee that a `.git/index.lock` file will be
+            # left over, and subsequent git operations will fail.
+            # We don't want that.
+            # As a result, we must rely on git to exit properly on SIGTERM.
+            proc.terminate()
+            # We wait() to ensure that git has finished before the next
+            # `gitstatus` prompt is rendered (otherwise `index.lock` still exists,
+            # and it will fail).
+            # We don't technically have to call `wait()` here as the
+            # `with subprocess.Popen()` context manager above would do that
+            # for us, but we do it to be explicit that waiting is being done.
+            proc.wait()  # we ignore what git says after we sent it SIGTERM
+            raise
 
 
 @xl.lazyobject
@@ -221,11 +243,10 @@ def _get_def(key):
 
 
 def _get_tag_or_hash():
-    tag = _check_output(['git', 'describe', '--exact-match']).strip()
-    if tag:
-        return tag
+    tag_or_hash = _check_output(['git', 'describe', '--always']).strip()
     hash_ = _check_output(['git', 'rev-parse', '--short', 'HEAD']).strip()
-    return _get_def('HASH') + hash_
+    have_tag_name = tag_or_hash != hash_
+    return tag_or_hash if have_tag_name else _get_def('HASH') + hash_
 
 
 def _get_stash(gitdir):
@@ -349,7 +370,7 @@ def _current_job():
 # vc
 #
 # -*- coding: utf-8 -*-
-"""Prompt formatter for simple version control branchs"""
+"""Prompt formatter for simple version control branches"""
 # pylint:disable=no-member, invalid-name
 
 # amalgamated os
@@ -360,10 +381,11 @@ threading = _LazyModule.load('threading', 'threading')
 # amalgamated subprocess
 # amalgamated xonsh.tools
 def _get_git_branch(q):
+    denv = builtins.__xonsh_env__.detype()
     try:
         branches = xt.decode_bytes(subprocess.check_output(
-            ['git', 'branch'],
-            stderr=subprocess.DEVNULL
+            ['git', 'branch'], env=denv,
+            stderr=subprocess.DEVNULL,
         )).splitlines()
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
@@ -500,13 +522,15 @@ def current_branch():
 
 def _git_dirty_working_directory(q, include_untracked):
     status = None
+    denv = builtins.__xonsh_env__.detype()
     try:
         cmd = ['git', 'status', '--porcelain']
         if include_untracked:
             cmd.append('--untracked-files=normal')
         else:
             cmd.append('--untracked-files=no')
-        status = subprocess.check_output(cmd, stderr=subprocess.DEVNULL)
+        status = subprocess.check_output(cmd, stderr=subprocess.DEVNULL,
+                                         env=denv)
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
     if status is not None:
@@ -557,9 +581,9 @@ def dirty_working_directory():
     """
     dwd = None
     cmds = builtins.__xonsh_commands_cache__
-    if cmds.lazy_locate_binary('git'):
+    if cmds.lazy_locate_binary('git', ignore_alias=True):
         dwd = git_dirty_working_directory()
-    if cmds.lazy_locate_binary('hg') and dwd is None:
+    if cmds.lazy_locate_binary('hg', ignore_alias=True) and dwd is None:
         dwd = hg_dirty_working_directory()
     return dwd
 
@@ -581,7 +605,7 @@ def branch_color():
 
 def branch_bg_color():
     """Return red if the current branch is dirty, yellow if the dirtiness can
-    not be determined, and green if it clean. These are bacground colors.
+    not be determined, and green if it clean. These are background colors.
     """
     dwd = dirty_working_directory()
     if dwd is None:
@@ -660,7 +684,7 @@ class PromptFormatter:
             val = self._get_field_value(field)
             return _format_value(val, spec, conv)
         else:
-            # color or unkown field, return as is
+            # color or unknown field, return as is
             return '{' + field + '}'
 
     def _get_field_value(self, field):
@@ -747,7 +771,7 @@ def multiline_prompt(curr=''):
     headlen = len(head)
     # tail is the trailing whitespace
     tail = line if headlen == 0 else line.rsplit(head[-1], 1)[1]
-    # now to constuct the actual string
+    # now to construct the actual string
     dots = builtins.__xonsh_env__.get('MULTILINE_PROMPT')
     dots = dots() if callable(dots) else dots
     if dots is None or len(dots) == 0:
